@@ -1,38 +1,40 @@
 /*
-* Copyright (c) 1997 - 2016
-* Actelion Pharmaceuticals Ltd.
-* Gewerbestrasse 16
-* CH-4123 Allschwil, Switzerland
-*
-* All rights reserved.
-*
-* Redistribution and use in source and binary forms, with or without
-* modification, are permitted provided that the following conditions are met:
-*
-* 1. Redistributions of source code must retain the above copyright notice, this
-*    list of conditions and the following disclaimer.
-* 2. Redistributions in binary form must reproduce the above copyright notice,
-*    this list of conditions and the following disclaimer in the documentation
-*    and/or other materials provided with the distribution.
-* 3. Neither the name of the the copyright holder nor the
-*    names of its contributors may be used to endorse or promote products
-*    derived from this software without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
-* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*
-*/
+ * Copyright (c) 1997 - 2016
+ * Actelion Pharmaceuticals Ltd.
+ * Gewerbestrasse 16
+ * CH-4123 Allschwil, Switzerland
+ *
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice, this
+ *    list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
+ * 3. Neither the name of the the copyright holder nor the
+ *    names of its contributors may be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
+ * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+ * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * @author Thomas Sander
+ */
 
 package com.actelion.research.chem;
 
+import com.actelion.research.gui.generic.GenericPolygon;
 import com.actelion.research.util.ColorHelper;
 
 import java.awt.*;
@@ -41,7 +43,7 @@ import java.awt.geom.Rectangle2D;
 import java.util.ArrayList;
 import java.util.Arrays;
 
-public abstract class AbstractDepictor {
+public abstract class AbstractDepictor<T> {
 	/*
 	 * If displayMode includes cDModeColorizeAtomLabels, then all atom labels are drawn
 	 * in a default color, which may be overridden by explicit atom colors or atom error colors.
@@ -138,6 +140,7 @@ public abstract class AbstractDepictor {
 	public static final int	cDModeNoImplicitAtomLabelColors = 0x0800;
 	public static final int	cDModeNoStereoProblem = 0x1000;
 	public static final int	cDModeNoColorOnESRAndCIP = 0x2000;
+	public static final int cDModeNoImplicitHydrogen = 0x4000;
 
 	private static final double cFactorTextSize = 0.6;
 	private static final double cFactorChiralTextSize = 0.5;
@@ -160,13 +163,14 @@ public abstract class AbstractDepictor {
 	private Rectangle2D.Double		mBoundingRect = new Rectangle2D.Double();
 	private DepictorTransformation	mTransformation;
 	private Point2D.Double			mChiralTextLocation;
-	private int[]					mAtomColor;
+	private int[]					mAtomColor,mAtomHiliteColor;
+	private float[]                 mAtomHiliteRadius;
 	private String[]				mAtomText;
 	private Point2D.Double[]		mAlternativeCoords;
 	private Color					mOverruleForeground,mOverruleBackground,mBondBGHiliteColor,mBondFGHiliteColor,
 									mExcludeGroupFGColor,mExcludeGroupBGColor,mCustomForeground,mCustomBackground,
 									mRGBColor;
-	protected Object				mG;
+	protected T                     mContext;
 
 	public AbstractDepictor(StereoMolecule mol) {
 		this(mol, 0);
@@ -196,13 +200,6 @@ public abstract class AbstractDepictor {
 		}
 
 
-/*	@Deprecated
-	public void setDefaultColor(int c) {
-		mStandardForegroundColor = c;
-	    updateBondHiliteColor();
-		}*/
-
-
 	/**
 	 * If the foreground color is set, the molecule is drawn in the foreground
 	 * color except for non carbon atoms, which are drawn in atomicNo specific
@@ -233,6 +230,18 @@ public abstract class AbstractDepictor {
 		}
 
 
+	/**
+	 * If you want this tructure view to also draw an atom background with specific colors for every atom,
+	 * then you need to call this method before calling paint().
+	 * @param argb values with a==0 are not considered
+	 * @param radius <= 1.0; if null, then a default of 0.5 of the average bond length is used
+	 */
+	public void setAtomHighlightColors(int[] argb, float[] radius) {
+		mAtomHiliteColor = argb;
+		mAtomHiliteRadius = radius;
+		}
+
+
 	public void setTransformation(DepictorTransformation t) {
 		mTransformation = t;
 		}
@@ -264,13 +273,13 @@ public abstract class AbstractDepictor {
 	 * Returns full transformation that moves/scales original molecule into viewRect.
 	 * This method considers atom labels when generating the bounding box. This includes
 	 * atom labels of drawn implicit hydrogen atoms.
-	 * @param g
+	 * @param context
 	 * @param viewRect
 	 * @param mode is typically (cModeInflateToMaxAVBL | maximum_desired_bond_length)
 	 * @return
 	 */
-	public DepictorTransformation updateCoords(Graphics g, Rectangle2D.Double viewRect, int mode) {
-		validateView(g, viewRect, mode);
+	public DepictorTransformation updateCoords(T context, Rectangle2D.Double viewRect, int mode) {
+		validateView(context, viewRect, mode);
 		if (mTransformation.isVoidTransformation()) {
 			return null;
 			}
@@ -305,18 +314,23 @@ public abstract class AbstractDepictor {
 		}
 
 
+	public StereoMolecule getMolecule() {
+		return mMol;
+		}
+
+
 	/**
 	 * A depictor maintains a DepictorTransformation object, which defines translation and scaling
 	 * of molecule coordinates into the viewRect. This method updates the depictor's transformation
 	 * such that the molecule is centered in viewRect and reduced in size if it doesn't fit.
 	 * If mode contains cModeInflateToMaxAVBL, then it is scaled to reach the desired mean bond length,
 	 * or to fill viewRect, whatever is reached first.
-	 * @param g
+	 * @param context
 	 * @param viewRect
 	 * @param mode (<0> or cModeInflateToMaxAVBL) + <desired mean bond length>
 	 * @return incremental transformation being applied to depictor's current transformation 
 	 */
-	public DepictorTransformation validateView(Object g, Rectangle2D.Double viewRect, int mode) {
+	public DepictorTransformation validateView(T context, Rectangle2D.Double viewRect, int mode) {
 		if (mMol.getAllAtoms() == 0)
 			return null;
 
@@ -328,7 +342,7 @@ public abstract class AbstractDepictor {
 		mpDot.clear();
 		mpTabuZone.clear();
 
-	    mG = g;
+	    mContext = context;
 		calculateParameters();
 
 		mpSetNormalLabelSize();
@@ -364,7 +378,7 @@ public abstract class AbstractDepictor {
 		if (mMol.getAllAtoms() == 0)
 			return null;
 
-		simpleCalculateBounds();
+		mBoundingRect = simpleCalculateBounds();
 
 		double avbl = mTransformation.getScaling() * mMol.getAverageBondLength();
 		DepictorTransformation t = new DepictorTransformation(mBoundingRect, viewRect, avbl, mode);
@@ -395,7 +409,7 @@ public abstract class AbstractDepictor {
     }
 
 
-    private void simpleCalculateBounds() {
+    public Rectangle2D.Double simpleCalculateBounds() {
 	    double minx = getAtomX(0);	// determine size of molecule
 	    double maxx = getAtomX(0);
 	    double miny = getAtomY(0);
@@ -408,7 +422,7 @@ public abstract class AbstractDepictor {
 			if (maxy < getAtomY(i)) maxy = getAtomY(i);
 			}
 
-		mBoundingRect = new Rectangle2D.Double(minx, miny, maxx-minx, maxy-miny);
+		return new Rectangle2D.Double(minx, miny, maxx-minx, maxy-miny);
 		}
 
 
@@ -480,19 +494,29 @@ public abstract class AbstractDepictor {
 		}
 
 
-	private double getAtomX(int atom) {
+	/**
+	 * @param atom
+	 * @return atom x-coordinate in Depictor's display space
+	 */
+	public double getAtomX(int atom) {
 		return mTransformation.transformX(mMol.getAtomX(atom));
 		}
 
 
-	private double getAtomY(int atom) {
+	/**
+	 * @param atom
+	 * @return atom y-coordinate in Depictor's display space
+	 */
+	public double getAtomY(int atom) {
 		return mTransformation.transformY(mMol.getAtomY(atom));
 		}
 
 
+	/**
+	 * Requires a call of updateCoords() or validateView() before calling this method.
+	 * @return the bounding rectangle in device coordinates (of the moved/scaled molecule)
+	 */
 	public final Rectangle2D.Double getBoundingRect() {
-			// requires a prior call of updateCoords() or validateView()
-			// returns the bounding rectangle in device coordinates (of the moved/scaled molecule)
 		return mBoundingRect;
 		}
 
@@ -533,16 +557,17 @@ public abstract class AbstractDepictor {
 		mChiralTextSize = averageBondLength * cFactorChiralTextSize + 0.5f;
 		}
 
-	public synchronized void paint(Object g) {
+
+	public synchronized void paint(T context) {
 		if (mMol.getAllAtoms() == 0)
 		    return;
 
 		mMol.ensureHelperArrays(requiredHelperArrays());
 
-		mG = g;
+		mContext = context;
 		calculateParameters();
 
-		int[][] esrGroupMemberCount = mMol.getERSGroupMemberCounts();
+		int[][] esrGroupMemberCount = mMol.getESRGroupMemberCounts();
 
 		boolean explicitAtomColors = false;
 		mAtomColor = new int[mMol.getAllAtoms()];
@@ -558,6 +583,9 @@ public abstract class AbstractDepictor {
 
 		setColor(COLOR_INITIALIZE);	// to initialize the color tracking mechanism
 
+		if (mAtomHiliteColor != null && (mAtomHiliteColor.length >= mMol.getAtoms()))
+			hiliteAtomBackgrounds(mAtomHiliteColor, mAtomHiliteRadius);
+
 		hiliteExcludeGroups();
 		hiliteBondBackgrounds();
 		indicateQueryFeatures();
@@ -590,7 +618,7 @@ public abstract class AbstractDepictor {
 				  && ((mDisplayMode & cDModeNoImplicitAtomLabelColors) == 0)
 				  && mMol.getAtomList(i) == null
 				  && mMol.getAtomicNo(i) < ATOM_LABEL_COLOR.length) {
-				setRGBColor(getContrastColor(ATOM_LABEL_COLOR[mMol.getAtomicNo(i)]));
+				setRGBColor(getContrastColor(ATOM_LABEL_COLOR[mMol.getAtomicNo(i)], i));
 	    		mpDrawAtom(i, esrGroupMemberCount);
 				setColor(mStandardForegroundColor);
 				}
@@ -604,9 +632,15 @@ public abstract class AbstractDepictor {
 		}
 
 
-	private Color getContrastColor(int rgb) {
+	public Color getBackgroundColor() {
+		return mOverruleBackground != null ? mOverruleBackground : mCustomBackground != null ? mCustomBackground : Color.WHITE;
+		}
+
+
+	private Color getContrastColor(int rgb, int atom) {
 		Color bg = (mOverruleBackground != null) ? mOverruleBackground
-				 : (mCustomBackground != null) ? mCustomBackground : Color.WHITE;
+				: (mAtomHiliteColor != null && atom < mAtomHiliteColor.length && (mAtomHiliteColor[atom] & 0xFF000000) != 0) ? new Color(mAtomHiliteColor[atom])
+				: (mCustomBackground != null) ? mCustomBackground : Color.WHITE;
 		Color fg = new Color(rgb);
 		return ColorHelper.getContrastColor(fg, bg);
 		}
@@ -650,7 +684,7 @@ public abstract class AbstractDepictor {
 		if (chiralText != null) {
 			if (mChiralTextLocation.x == 0.0 && mChiralTextLocation.y == 0.0) {
 				double avbl = mTransformation.getScaling() * mMol.getAverageBondLength();
-				simpleCalculateBounds();
+				mBoundingRect = simpleCalculateBounds();
 				expandBoundsByTabuZones(avbl);
 				setChiralTextLocation(null, avbl, 0);
 				}
@@ -663,6 +697,30 @@ public abstract class AbstractDepictor {
 		}
 
 
+	/**
+	 * May be overridden to get a more appealing highlighting than just round circles
+	 * @param argb if alpha < 1 then the background is mixed in accordingly
+	 * @param radius <= 1.0; if null, then a default of 0.5 of the average bond length is used
+	 */
+	public void hiliteAtomBackgrounds(int[] argb, float[] radius) {
+		Color background = (mOverruleBackground != null) ? mOverruleBackground
+				: (mCustomBackground != null) ? mCustomBackground : Color.WHITE;
+
+		double avbl = mTransformation.getScaling() * mMol.getAverageBondLength();
+		for (int atom=0; atom<mMol.getAtoms(); atom++) {
+			int alpha = (argb[atom] & 0xFF000000) >> 24;
+			if (alpha != 0) {
+				Color color = new Color(argb[atom]);
+				if (alpha != 255)
+					color = ColorHelper.intermediateColor(background, color, (float)alpha/255f);
+				double r = (radius == null) ? 0.5 * avbl : 0.6 * radius[atom] * avbl;
+				setColor(color);
+	            fillCircle(getAtomX(atom)-r, getAtomY(atom)-r, 2*r);
+				}
+			}
+		}
+
+
 	private void hiliteBondBackgrounds() {
         setLineWidth(2*mpBondHiliteRadius);
         DepictorLine line = new DepictorLine();
@@ -813,10 +871,6 @@ public abstract class AbstractDepictor {
 		int atom1 = mMol.getBondAtom(0,bnd);
 		int atom2 = mMol.getBondAtom(1,bnd);
 
-		boolean isExcludeGroup = ((mMol.getAtomQueryFeatures(atom1)
-								 | mMol.getAtomQueryFeatures(atom2))
-								  & Molecule.cAtomQFExcludeGroup) != 0;
-
         onDrawBond(bnd,getAtomX(atom1),getAtomY(atom1),getAtomX(atom2),getAtomY(atom2));
 
 		// if one of the bond atoms is part of an exclude group
@@ -856,7 +910,23 @@ public abstract class AbstractDepictor {
 
 		switch (bondOrder) {
 		case 1:
-            switch (mMol.getBondType(bnd)) {
+			int bondType = mMol.getBondType(bnd);
+			if ((mDisplayMode & cDModeSuppressESR) != 0
+			 && (bondType == Molecule.cBondTypeUp || bondType == Molecule.cBondTypeDown)) {
+				int stereoCenter = mMol.getBondAtom(0, bnd);
+				int esrType = mMol.getAtomESRType(stereoCenter);
+				if (esrType != Molecule.cESRTypeAbs) {
+					int esrGroup = mMol.getAtomESRGroup(stereoCenter);
+					int count = 0;
+					for (int atom=0; atom<mMol.getAtoms(); atom++)
+						if (mMol.getAtomESRType(atom) == esrType
+						 && mMol.getAtomESRGroup(atom) == esrGroup)
+							count++;
+					if (count == 1)
+						bondType = Molecule.cBondTypeSingle;
+					}
+				}
+            switch (bondType) {
 			case Molecule.cBondTypeSingle:
 				mpHandleLine(theLine, atom1, atom2);
 				break;
@@ -1577,7 +1647,6 @@ public abstract class AbstractDepictor {
         if (!mIsValidatingView)
             onDrawAtom(atom,mMol.getAtomLabel(atom), getAtomX(atom), getAtomY(atom));
 
-
 		String propStr = null;
 		if (mMol.getAtomCharge(atom) != 0) {
 			String valStr = (Math.abs(mMol.getAtomCharge(atom)) == 1) ? ""
@@ -1596,8 +1665,6 @@ public abstract class AbstractDepictor {
 				isoStr = append(isoStr, "!a");
 			if ((queryFeatures & Molecule.cAtomQFMoreNeighbours) != 0)
 				isoStr = append(isoStr, "s");
-//			if ((queryFeatures & Molecule.cAtomQFNoMoreNeighbours) != 0)
-//				isoStr = append(isoStr, "!s");
             if ((queryFeatures & Molecule.cAtomQFHydrogen) != 0) {
                 int hydrogens = (queryFeatures & Molecule.cAtomQFHydrogen);
     			if (hydrogens == Molecule.cAtomQFNot1Hydrogen+Molecule.cAtomQFNot2Hydrogen+Molecule.cAtomQFNot3Hydrogen)
@@ -1749,21 +1816,18 @@ public abstract class AbstractDepictor {
             }
 
 		int hydrogensToAdd = 0;
-		if (mMol.isFragment()) {
-/*			if ((mMol.getAtomicNo(atom) != 6
-			  || !mAtomIsConnected[atom]
-			  || mMol.getAtomCharge(atom) != 0
-			  || mMol.getAtomRadical(atom) != 0)
-			 && (mMol.getAtomQueryFeatures(atom) & Molecule.cAtomQFNoMoreNeighbours) != 0)*/
-			if ((mMol.getAtomQueryFeatures(atom) & Molecule.cAtomQFNoMoreNeighbours) != 0)
-				hydrogensToAdd = mMol.getImplicitHydrogens(atom);
-			}
-		else {
-			if (mMol.getAtomicNo(atom) != 6
-			 || mMol.getAtomMass(atom) != 0
-			 || !mAtomIsConnected[atom]
-			 || mMol.getAtomRadical(atom) != 0)
-				hydrogensToAdd = mMol.getImplicitHydrogens(atom);
+		if ((mDisplayMode & cDModeNoImplicitHydrogen) == 0) {
+			if (mMol.isFragment()) {
+				if ((mMol.getAtomQueryFeatures(atom) & Molecule.cAtomQFNoMoreNeighbours) != 0)
+					hydrogensToAdd = mMol.getImplicitHydrogens(atom);
+				}
+			else {
+				if (mMol.getAtomicNo(atom) != 6
+				 || mMol.getAtomMass(atom) != 0
+				 || !mAtomIsConnected[atom]
+				 || mMol.getAtomRadical(atom) != 0)
+					hydrogensToAdd = mMol.getImplicitHydrogens(atom);
+				}
 			}
 
 		boolean largeIsoString = false;
@@ -2324,29 +2388,26 @@ public abstract class AbstractDepictor {
 
 
 	private void drawWedge(DepictorLine theWedge,int atom1, int atom2) {
-		double p1x[],p1y[],p2x[],p2y[];
-		double xdiff,ydiff;
+		double xdiff = (theWedge.y1 - theWedge.y2) / 9;
+		double ydiff = (theWedge.x2 - theWedge.x1) / 9;
+		double xe1 = (theWedge.x2 + xdiff);
+		double ye1 = (theWedge.y2 + ydiff);
+		double xe2 = (theWedge.x2 - xdiff);
+		double ye2 = (theWedge.y2 - ydiff);
+		double xm1 = (theWedge.x1 + xe1) / 2;
+		double ym1 = (theWedge.y1 + ye1) / 2;
+		double xm2 = (theWedge.x1 + xe2) / 2;
+		double ym2 = (theWedge.y1 + ye2) / 2;
 
-		xdiff = (theWedge.y1 - theWedge.y2) / 9;
-		ydiff = (theWedge.x2 - theWedge.x1) / 9;
-		p1x = new double[3];
-		p1y = new double[3];
-		p2x = new double[4];
-		p2y = new double[4];
-		p1x[0] = theWedge.x1;
-		p1y[0] = theWedge.y1;
-		p2x[2] = (theWedge.x2 + xdiff);
-		p2y[2] = (theWedge.y2 + ydiff);
-		p2x[3] = (theWedge.x2 - xdiff);
-		p2y[3] = (theWedge.y2 - ydiff);
-		p1x[1] = (p1x[0] + p2x[2]) / 2;
-		p1y[1] = (p1y[0] + p2y[2]) / 2;
-		p1x[2] = (p1x[0] + p2x[3]) / 2;
-		p1y[2] = (p1y[0] + p2y[3]) / 2;
-		p2x[0] = p1x[2];
-		p2y[0] = p1y[2];
-		p2x[1] = p1x[1];
-		p2y[1] = p1y[1];
+		GenericPolygon p1 = new GenericPolygon(3);
+		GenericPolygon p2 = new GenericPolygon(4);
+		p1.addPoint(theWedge.x1, theWedge.y1);
+		p1.addPoint(xm1, ym1);
+		p1.addPoint(xm2, ym2);
+		p2.addPoint(xm2, ym2);
+		p2.addPoint(xm1, ym1);
+		p2.addPoint(xe1, ye1);
+		p2.addPoint(xe2, ye2);
 
 		int color1,color2;
 		if (mMol.isBondForegroundHilited(mMol.getBond(atom1, atom2))) {
@@ -2364,9 +2425,9 @@ public abstract class AbstractDepictor {
 			}
 
         setColor(color1);
-		drawPolygon(p1x,p1y,3);
+		drawPolygon(p1);
 		setColor(color2);
-		drawPolygon(p2x,p2y,4);
+		drawPolygon(p2);
 		setColor(mStandardForegroundColor);
 		}
 
@@ -2539,27 +2600,9 @@ public abstract class AbstractDepictor {
             }
         }
 
-/*	private void drawBlackArc(MyRect theArc,double arcAngle,boolean inverted) {
-		double xdif = (theArc.x2 - theArc.x1);
-		double ydif = (theArc.y2 - theArc.y1);
-		double length = Math.sqrt(xdif*xdif + ydif*ydif);
-		double theAngle = (inverted) ? (Math.PI - cArcAngle) / 2
-									 : (cArcAngle - Math.PI) / 2;
-		double radius = (length / 2) / Math.cos(theAngle);
-		double centerX = theArc.x1 + radius * Math.sin(arcAngle + theAngle);
-		double centerY = theArc.y1 + radius * Math.cos(arcAngle + theAngle);
-
-		mG.drawArc((int)(centerX - radius + 0.5),(int)(centerY - radius + 0.5),
-				   (int)(2 * radius + 0.5),(int)(2 * radius + 0.5),
-				   (int)((arcAngle - theAngle) * 180 / Math.PI) - 90,
-				   (inverted) ? (int)(-cArcAngle * 180 / Math.PI + 0.5)
-				              : (int)(cArcAngle * 180 / Math.PI + 0.5));
-		}	*/
-
-
 	protected abstract void drawBlackLine(DepictorLine theLine);
     protected abstract void drawDottedLine(DepictorLine theLine);
-	protected abstract void drawPolygon(double[] x, double[] y, int count);
+	protected abstract void drawPolygon(GenericPolygon p);
 	protected abstract void drawString(String theString,double x,double y);
 	protected abstract void fillCircle(double x, double y, double d);
 	protected abstract double getLineWidth();
