@@ -1,35 +1,36 @@
 /*
-* Copyright (c) 1997 - 2016
-* Actelion Pharmaceuticals Ltd.
-* Gewerbestrasse 16
-* CH-4123 Allschwil, Switzerland
-*
-* All rights reserved.
-*
-* Redistribution and use in source and binary forms, with or without
-* modification, are permitted provided that the following conditions are met:
-*
-* 1. Redistributions of source code must retain the above copyright notice, this
-*    list of conditions and the following disclaimer.
-* 2. Redistributions in binary form must reproduce the above copyright notice,
-*    this list of conditions and the following disclaimer in the documentation
-*    and/or other materials provided with the distribution.
-* 3. Neither the name of the the copyright holder nor the
-*    names of its contributors may be used to endorse or promote products
-*    derived from this software without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
-* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*
-*/
+ * Copyright (c) 1997 - 2016
+ * Actelion Pharmaceuticals Ltd.
+ * Gewerbestrasse 16
+ * CH-4123 Allschwil, Switzerland
+ *
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice, this
+ *    list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
+ * 3. Neither the name of the the copyright holder nor the
+ *    names of its contributors may be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
+ * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+ * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * @author Thomas Sander
+ */
 
 package com.actelion.research.chem;
 
@@ -54,7 +55,7 @@ public class SSSearcher {
 	pyrol but not indol!!!
   - match modes used for the actual atom by atom check must be more or equally restrictive
 	than the match mode used for index creation. Otherwise index keys may filter out
-	molecules which would be considered a match with the less strict matching consitions
+	molecules which would be considered a match with the less strict matching conditions
 	of the atom by atom check.
 */
 
@@ -62,11 +63,12 @@ public class SSSearcher {
 	public static final int cIndexMatchMode = cMatchDBondToDelocalized;
 	public static final int cDefaultMatchMode = cMatchAromDBondToDelocalized;
 
-	public final static int cCountModeExistance		= 1;
-	public final static int cCountModeFirstMatch	= 2;
-	public final static int cCountModeSeparated		= 3;
-	public final static int cCountModeOverlapping	= 4;
-	public final static int cCountModeRigorous		= 5;
+	public final static int cCountModeExistance		= 1;    // check only, don't create matchList
+	public final static int cCountModeFirstMatch	= 2;    // create matchList with just one match
+	public final static int cCountModeSeparated		= 3;    // create list of all non-overlapping matches / not optimized for maximum match count
+	public final static int cCountModeOverlapping	= 4;    // create list not containing multiple matches sharing exactly the same atoms
+	public final static int cCountModeRigorous		= 5;    // create list of all possible matches neglecting any symmetries
+	public final static int cCountModeUnique		= 6;    // create list of all distinguishable matches considering symmetries
 
 	// default behaviour for unusual atom masses and atom charges is that
 	// - if no atom charge/mass is specified in the query then all charges/masses match
@@ -100,6 +102,7 @@ public class SSSearcher {
 	private int[] mMatchTable;
 	private int[] mExcludeGroupNo;
 	private int[] mExcludeGroupGraphIndex;
+	private int[] mFragmentAtomContextRank;
 
 	// depending on the fragment count mode this may contain atom lists
 	// of all till now located matching sub-fragments
@@ -112,6 +115,8 @@ public class SSSearcher {
 	private int mRequiredHelperLevel;
 	private int mExcludeGroupCount;
 
+	private volatile boolean mStop;
+
 	/**
 	 * Instantiates a SSSearcher object for running sub-structure searches
 	 * with one or more sub-structure fragments on one or more molecules.
@@ -171,6 +176,12 @@ public class SSSearcher {
 		mMolecule.ensureHelperArrays(Molecule.cHelperNeighbours);
 		}
 
+	/**
+	 * Asks the substructure search to stop without completing as soon as possible.
+	 */
+	public void stop() {
+		mStop = true;
+		}
 
 	/**
 	 * Defines the fragment to be used in isFragmentInMolecule(...)
@@ -211,6 +222,8 @@ public class SSSearcher {
 		mExcludeGroupCount = 0;
 		mExcludeGroupNo = null;
 
+		mFragmentAtomContextRank = null;
+
 		if (mFragmentExcludeAtoms != 0) {
 			if (mFragmentExcludeAtoms != 0)
 				for (int bond = 0; bond < mFragment.getBonds(); bond++)
@@ -227,6 +240,22 @@ public class SSSearcher {
 		}
 
 
+	/**
+	 * If countMode is cCountModeUnique, then matches are considered distinct, if<br>
+	 * - either the list of matching molecule atoms to the query fragment is a different one<br>
+	 * - or if the mutual combination of fragment and molecule atom's symmetry rank is different
+	 *   (when the list of matched molecule atoms is the same)<br>
+	 * For certain situations fragment atoms must be considered different, even if their symmetry rank
+	 * is equal, e.g. in the context of a reaction where equivalent reactant atoms end up in different
+	 * product environments. This method allows to specify an additional criterion for the uniqueness
+	 * comparison to be considered. In case of reactions, these might be the symmetry ranks of the
+	 * products atoms mapped to the reactant atoms.
+	 * Note: The current implementation only uses the 8 least significant bits of the context rank.
+	 */
+	public void setFragmentSymmetryConstraints(int[] fragmentContextRank) {
+		mFragmentAtomContextRank = fragmentContextRank;
+		}
+
 	/**
 	 * Build a graph of the query fragment(s) including ring closures as redundant nodes.
 	 * If we have exclude groups then these are added at the end of the graph.
@@ -417,12 +446,13 @@ System.out.println();
 	 * are counted and listed only once. If count mode is different from cCountModeExistance,
 	 * then an atom mapping from fragment to molecule is collected and can be retrieved with getMatchList().
 	 * If the query fragment does not contain atoms other than exclude group atoms, then no match is returned.
-	 * @param countMode one of cCountModeExistance, cCountModeFirstMatch, cCountModeOverlapping, cCountModeRigorous
+	 * @param countMode one of cCountModeExistance, cCountModeFirstMatch, cCountModeSeparated, cCountModeOverlapping, cCountModeUnique, cCountModeRigorous
 	 * @param matchMode cDefaultMatchMode or combination of cMatchAtomCharge, cMatchAtomMass, cMatchDBondToDelocalized, cMatchAromDBondToDelocalized
 	 * @param atomExcluded defines atoms of molecule to be excluded from sub-structure matching
 	 * @return count of sub-structure matches of fragment in molecule
 	 */
 	public int findFragmentInMolecule(int countMode, int matchMode, final boolean[] atomExcluded) {
+		mStop = false;
 		mMatchList = new ArrayList<int[]>();
 		mSortedMatchSet.clear();
 
@@ -445,6 +475,9 @@ for (int j=0; j<moleculeAtoms; j++)
 System.out.println();
 */
 
+		if (countMode == cCountModeUnique)
+			mRequiredHelperLevel = Molecule.cHelperSymmetrySimple;
+
 		setupAtomAndBondFeatures(matchMode);
 
 		// atom usage mask in mMolecule
@@ -464,7 +497,7 @@ System.out.println();
 		// - otherwise the current connAtom index of the parent atom in the matching graph
 
 		int current = 0;
-		while (true) {
+		while (!mStop) {
 /*
 System.out.print("  index:"); for (int i=0; i<mFragmentGraphSize; i++) System.out.print(" "+(index[i]==-1?"-":""+index[i])); System.out.println();
 System.out.print("		"); for (int i=0; i<current; i++) System.out.print("  "); System.out.println(" ^");
@@ -560,53 +593,50 @@ System.out.println();
 
 
 	private void addMatchIfQualifies(int countMode) {
-		int[] match = copyOf(mMatchTable, mMatchTable.length);
-
 		if (countMode == cCountModeFirstMatch
 		 || countMode == cCountModeRigorous) {
 			// count every match (even permutations of same atoms)
-			mMatchList.add(match);
-			return;
+			mMatchList.add(copyOf(mMatchTable, mMatchTable.length));
 			}
-
-		if (mFragmentExcludeAtoms != 0	// store matches as indication that we have found something
-		 || countMode == cCountModeOverlapping
-		 || countMode == cCountModeSeparated) {
-			match = getSortedMatch(match);
-			if (countMode == cCountModeOverlapping) {
-				if (!mSortedMatchSet.contains(match)) {
-					mSortedMatchSet.add(match);
+		else if (countMode == cCountModeOverlapping) {
+			int[] sortedMatch = getSortedMatch(copyOf(mMatchTable, mMatchTable.length));
+			if (!mSortedMatchSet.contains(sortedMatch)) {
+				mSortedMatchSet.add(sortedMatch);
+				mMatchList.add(copyOf(mMatchTable, mMatchTable.length));
+				}
+			}
+		else if (countMode == cCountModeSeparated) {
+			int[] sortedMatch = getSortedMatch(copyOf(mMatchTable, mMatchTable.length));
+			if (!mSortedMatchSet.contains(sortedMatch)) {
+				boolean found = false;
+				for (int[] existing:mSortedMatchSet) {
+					int existingIndex = 0;
+					for (int atom:sortedMatch) {
+						while (existingIndex < existing.length && existing[existingIndex] < atom)
+							existingIndex++;
+						if (existingIndex < existing.length) {
+							if (atom == existing[existingIndex]) {
+								found = true;
+								break;
+								}
+							}
+						}
+					if (found)
+						break;
+					}
+				if (!found) {
+					mSortedMatchSet.add(sortedMatch);
 					mMatchList.add(copyOf(mMatchTable, mMatchTable.length));
 					}
 				}
-			else if (countMode == cCountModeSeparated) {
-				if (!mSortedMatchSet.contains(match)) {
-					boolean found = false;
-					for (int[] existing:mSortedMatchSet) {
-						int existingIndex = 0;
-						for (int atom:match) {
-							while (existingIndex < existing.length && existing[existingIndex] < atom)
-								existingIndex++;
-							if (existingIndex < existing.length) {
-								if (atom == existing[existingIndex]) {
-									found = true;
-									break;
-									}
-								}
-							}
-						if (found)
-							break;
-						}
-					if (!found) {
-						mSortedMatchSet.add(match);
-						mMatchList.add(copyOf(mMatchTable, mMatchTable.length));
-						}
-					}
-				}
-			return;
 			}
-
-		return;
+		else if (countMode == cCountModeUnique) {
+			int[] sortedMatch = getSortedSymmetryMatch(copyOf(mMatchTable, mMatchTable.length));
+			if (!mSortedMatchSet.contains(sortedMatch)) {
+				mSortedMatchSet.add(sortedMatch);
+				mMatchList.add(copyOf(mMatchTable, mMatchTable.length));
+				}
+			}
 		}
 
 
@@ -632,6 +662,30 @@ System.out.println();
 		return match;
 		}
 
+	/**
+	 * @return sorted match atoms without excluded atoms
+	 */
+	private int[] getSortedSymmetryMatch(int[] match) {
+		int count = 0;
+		for (int atom:match)
+			if (atom == -1)
+				count++;
+
+		int[] symmetryMatch = new int[match.length - count];
+		int index = 0;
+		for (int i=0; i<match.length; i++) {
+			if (match[i] != -1) {
+				symmetryMatch[index] = (mFragment.getSymmetryRank(i) << 16) | mMolecule.getSymmetryRank(match[i]);
+				if (mFragmentAtomContextRank != null)
+					symmetryMatch[index] |= mFragmentAtomContextRank[i] << 24;
+				index++;
+				}
+			}
+
+		Arrays.sort(symmetryMatch);
+		return symmetryMatch;
+		}
+
 	public boolean areAtomsSimilar(int moleculeAtom, int fragmentAtom) {
 		int moleculeConnAtoms = mMolecule.getConnAtoms(moleculeAtom);
 		int fragmentConnAtoms = mFragmentConnAtoms[fragmentAtom];
@@ -716,6 +770,9 @@ System.out.println();
 		if (mFragment.getAtomMass(fragmentAtom) != 0
 		 && mFragment.getAtomMass(fragmentAtom) != mMolecule.getAtomMass(moleculeAtom))
 			return false;
+		if (mFragment.getAtomRadical(fragmentAtom) != 0
+		 && mFragment.getAtomRadical(fragmentAtom) != mMolecule.getAtomRadical(moleculeAtom))
+			return false;
 		int ringSize = (mFragment.getAtomQueryFeatures(fragmentAtom) & Molecule.cAtomQFRingSize) >> Molecule.cAtomQFRingSizeShift;
 		if (ringSize != 0) {
 			if (mMolecule.isFragment()
@@ -763,7 +820,7 @@ System.out.println();
 				if (fragmentParity == Molecule.cAtomParityNone)
 			   		continue;
 
-				// consider as match if assymetric fragment atom matches on non-stereo-center
+				// consider as match if asymetrical fragment atom matches on non-stereo-center
 				if (moleculeParity == Molecule.cAtomParityNone)
 			   		continue;
 
@@ -775,7 +832,7 @@ System.out.println();
 			   		continue;
 
 				if (moleculeParity == Molecule.cAtomParityUnknown)
-			   		continue;
+			   		return false;
 
 				if (mFragment.getAtomESRType(fragmentAtom) == Molecule.cESRTypeAnd) {
 					esrGroupAtomCount++;
