@@ -511,6 +511,56 @@ public class ObjectiveBlurFlexophoreHardMatchUncovered implements IObjectiveComp
 
 	}
 
+	/**
+	 *
+	 * @param solution a valid solution for IMolDistHis base and IMolDistHis query. Query and base must be set before
+	 *                 starting the similarity calculation.
+	 * @param indexHeap heap index of the node for which the histogram similarities to all other nodes will be calculated.
+	 *                  indexHeap is the index from the list of matching query and base nodes.
+	 * @return histogram similarity for a single node. Calculated as average from the sum of histogram similarities.
+	 */
+	public float getSimilarityHistogramsForNode(SolutionCompleteGraph solution, int indexHeap) {
+		int heap = solution.getSizeHeap();
+
+		//
+		// the query must hit with all pharmacophore nodes
+		//
+		if(modeQuery) {
+			if (nodesQuery != heap) {
+				return 0;
+			}
+		}
+
+		double sumPairwiseMapping = 0;
+
+		int indexNode1Query = solution.getIndexQueryFromHeap(indexHeap);
+
+		int indexNode1Base = solution.getIndexCorrespondingBaseNode(indexNode1Query);
+
+		for (int i = 0; i < heap; i++) {
+			if(indexHeap==i)
+				continue;
+
+			int indexNode2Query = solution.getIndexQueryFromHeap(i);
+
+			int indexNode2Base = solution.getIndexCorrespondingBaseNode(indexNode2Query);
+
+			double simHists = getSimilarityHistogram(indexNode1Query, indexNode2Query, indexNode1Base, indexNode2Base);
+
+			sumPairwiseMapping += simHists;
+
+			if(verbose) {
+				System.out.println("scorePairwiseMapping " + Formatter.format2(simHists));
+			}
+		}
+
+		double mappings = heap-1;
+
+		double avrPairwiseMapping = sumPairwiseMapping/mappings;
+
+		return (float)avrPairwiseMapping;
+	}
+
 	public long getDeltaNanoQueryBlur() {
 		return deltaNanoQueryBlur;
 	}
@@ -568,8 +618,13 @@ public class ObjectiveBlurFlexophoreHardMatchUncovered implements IObjectiveComp
 
 		long t0 = System.nanoTime();
 
-		mdhvBase = (MolDistHistViz)iMolDistHistBase;
-		mdhvBaseBlurredHist = new MolDistHistViz((MolDistHistViz)iMolDistHistBase);
+		if(iMolDistHistBase instanceof MolDistHistViz) {
+			mdhvBase = (MolDistHistViz) iMolDistHistBase;
+			mdhvBaseBlurredHist = new MolDistHistViz((MolDistHistViz) iMolDistHistBase);
+		} else if(iMolDistHistBase instanceof MolDistHist) {
+			mdhvBase = new MolDistHistViz((MolDistHist) iMolDistHistBase);
+			mdhvBaseBlurredHist = new MolDistHistViz((MolDistHist) iMolDistHistBase);
+		}
 
 		slidingWindowDistHist.apply(mdhvBaseBlurredHist);
 
@@ -590,8 +645,13 @@ public class ObjectiveBlurFlexophoreHardMatchUncovered implements IObjectiveComp
 
 		long t0 = System.nanoTime();
 
-		mdhvQuery = (MolDistHistViz) iMolDistHistQuery;
-		mdhvQueryBlurredHist = new MolDistHistViz((MolDistHistViz)iMolDistHistQuery);
+		if(iMolDistHistQuery instanceof MolDistHistViz) {
+			mdhvQuery = (MolDistHistViz) iMolDistHistQuery;
+			mdhvQueryBlurredHist = new MolDistHistViz((MolDistHistViz) iMolDistHistQuery);
+		} else if(iMolDistHistQuery instanceof MolDistHist) {
+			mdhvQuery = new MolDistHistViz((MolDistHist) iMolDistHistQuery);
+			mdhvQueryBlurredHist = new MolDistHistViz((MolDistHist) iMolDistHistQuery);
+		}
 
 		slidingWindowDistHist.apply(mdhvQueryBlurredHist);
 
@@ -835,12 +895,10 @@ public class ObjectiveBlurFlexophoreHardMatchUncovered implements IObjectiveComp
 		
 		double simHists = getSimilarityHistogram(indexNode1Query, indexNode2Query, indexNode1Base, indexNode2Base);
 		
-		if(simHists==0){
-
-			System.out.println("ObjectiveFlexophoreHardMatchUncovered getScorePairwiseMapping(int indexNode1Query, int indexNode2Query, int indexNode1Base, int indexNode2Base)");
-
-			System.out.println("Sim hists = 0");
-		}
+//		if(simHists==0){
+//			System.out.println("ObjectiveFlexophoreHardMatchUncovered getScorePairwiseMapping(int indexNode1Query, int indexNode2Query, int indexNode1Base, int indexNode2Base)");
+//			System.out.println("Sim hists = 0");
+//		}
 
 		if(verbose){
 			System.out.println("simHists " + Formatter.format2(simHists));
@@ -854,7 +912,7 @@ public class ObjectiveBlurFlexophoreHardMatchUncovered implements IObjectiveComp
 		return score;
 	}
 
-	public float getSimilarityNodes(int indexNodeQuery, int indexNodeBase) {
+	public double getSimilarityNodes(int indexNodeQuery, int indexNodeBase) {
 		
 		if(arrSimilarityNodes[indexNodeQuery][indexNodeBase] < 0 || verbose){
 			
