@@ -1,35 +1,36 @@
 /*
-* Copyright (c) 1997 - 2016
-* Actelion Pharmaceuticals Ltd.
-* Gewerbestrasse 16
-* CH-4123 Allschwil, Switzerland
-*
-* All rights reserved.
-*
-* Redistribution and use in source and binary forms, with or without
-* modification, are permitted provided that the following conditions are met:
-*
-* 1. Redistributions of source code must retain the above copyright notice, this
-*    list of conditions and the following disclaimer.
-* 2. Redistributions in binary form must reproduce the above copyright notice,
-*    this list of conditions and the following disclaimer in the documentation
-*    and/or other materials provided with the distribution.
-* 3. Neither the name of the the copyright holder nor the
-*    names of its contributors may be used to endorse or promote products
-*    derived from this software without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
-* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*
-*/
+ * Copyright (c) 1997 - 2016
+ * Actelion Pharmaceuticals Ltd.
+ * Gewerbestrasse 16
+ * CH-4123 Allschwil, Switzerland
+ *
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice, this
+ *    list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
+ * 3. Neither the name of the the copyright holder nor the
+ *    names of its contributors may be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
+ * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+ * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * @author Thomas Sander
+ */
 
 package com.actelion.research.chem.io;
 
@@ -177,7 +178,7 @@ public class DWARFileParser extends CompoundFileParser implements DescriptorCons
 	    setDescriptorHandlerFactory(DescriptorHandlerStandard2DFactory.getFactory());
 
     	if ((mMode & MODE_BUFFER_HEAD_AND_TAIL) != 0)
-    		mHeadOrTailLineList = new ArrayList<String>();
+    		mHeadOrTailLineList = new ArrayList<>();
 
     	int coordinateMode = mMode & MODE_COORDINATE_MASK;
 	
@@ -210,7 +211,7 @@ public class DWARFileParser extends CompoundFileParser implements DescriptorCons
 
         line = readHeadOrTailLine();
 
-        if (line != null
+        while (line != null
          && (line.equals(cFileExplanationStart)
           || line.equals(cMacroListStart))) {
             line = readHeadOrTailLine();
@@ -221,7 +222,7 @@ public class DWARFileParser extends CompoundFileParser implements DescriptorCons
             line = readHeadOrTailLine();
         	}
 
-        mColumnPropertyMap = new TreeMap<String,Properties>();
+        mColumnPropertyMap = new TreeMap<>();
 
         if (line != null
          && line.equals(cColumnPropertyStart)) {
@@ -246,7 +247,7 @@ public class DWARFileParser extends CompoundFileParser implements DescriptorCons
             line = readHeadOrTailLine();
             }
 
-        mSpecialFieldMap = new TreeMap<String,SpecialField>();	// only take those columns that have a special type
+        mSpecialFieldMap = new TreeMap<>();	// only take those columns that have a special type
         for (String columnName:mColumnPropertyMap.keySet()) {
         	Properties properties = mColumnPropertyMap.get(columnName);
         	String specialType = properties.getProperty(cColumnPropertySpecialType);
@@ -434,7 +435,7 @@ public class DWARFileParser extends CompoundFileParser implements DescriptorCons
      * returns a list of all header/footer rows of the DWAR file. If this method is
      * called before all rows have been read, then the header lines including column
      * properties and the column title line are returned. If this method is
-     * called before all rows have been read, then all lines after the data table, i.e. the
+     * called after all rows have been read, then all lines after the data table, i.e. the
      * runtime properties, are returned.
      * @return
      */
@@ -495,13 +496,13 @@ public class DWARFileParser extends CompoundFileParser implements DescriptorCons
 		int index2 = mLine.indexOf('\t');
 		while (index2 != -1) {
 			if (column<mFieldData.length)
-				mFieldData[column] = mLine.substring(index1, index2).replace(NEWLINE_STRING, cLineSeparator);
+				mFieldData[column] = mLine.substring(index1, index2).replace(NEWLINE_STRING, cLineSeparator).replace(TAB_STRING, "\t");
 			column++;
 			index1 = index2+1;
 			index2 = mLine.indexOf('\t', index1);
 			}
 		if (column<mFieldData.length)
-			mFieldData[column] = mLine.substring(index1).replace(NEWLINE_STRING, cLineSeparator);
+			mFieldData[column] = mLine.substring(index1).replace(NEWLINE_STRING, cLineSeparator).replace(TAB_STRING, "\t");
 
 		return true;
 		}
