@@ -1,17 +1,33 @@
 /*
- * Copyright 2017 Idorsia Pharmaceuticals Ltd., Hegenheimermattweg 91, CH-4123 Allschwil, Switzerland
+ * Copyright (c) 1997 - 2016
+ * Actelion Pharmaceuticals Ltd.
+ * Gewerbestrasse 16
+ * CH-4123 Allschwil, Switzerland
  *
- * This file is part of DataWarrior.
- * 
- * DataWarrior is free software: you can redistribute it and/or modify it under the terms of the
- * GNU General Public License as published by the Free Software Foundation, either version 3 of
- * the License, or (at your option) any later version.
- * 
- * DataWarrior is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License along with DataWarrior.
- * If not, see http://www.gnu.org/licenses/.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice, this
+ *    list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
+ * 3. Neither the name of the the copyright holder nor the
+ *    names of its contributors may be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
+ * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+ * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  * @author Thomas Sander
  */
@@ -429,12 +445,12 @@ public class CompoundCollectionPane<T> extends JScrollPane
 							}
 
 						Depictor2D d = new Depictor2D(compound, mDisplayMode);
-						d.validateView(g,
+						d.validateView((Graphics2D)g,
 									   new Rectangle2D.Double(bounds.x, bounds.y, bounds.width, bounds.height),
 									   AbstractDepictor.cModeInflateToMaxAVBL);
 
 						d.setForegroundColor(foreground, background);
-						d.paint(g);
+						d.paint((Graphics2D)g);
 
 						if (mSelectedIndex == i || mHighlightedIndex == i) {
 							g.setColor(!mIsEnabled ? ColorHelper.getContrastColor(Color.GRAY, background)
@@ -603,12 +619,14 @@ public class CompoundCollectionPane<T> extends JScrollPane
 		}
 
 	public void structureChanged(StereoMolecule mol) {
+		String reason = (mCompoundFilter == null) ? null
+				: (mCompoundFilter instanceof SubstructureFilter) ? "match the substructure" : "qualify";
 		if (mEditedIndex == -1) {	// new structure
 			if (mol.getAllAtoms() != 0) {
 				if (mCompoundFilter == null || mCompoundFilter.moleculeQualifies(mol))
 					mModel.addMolecule(mModel.getSize(), mol);
 				else
-					JOptionPane.showMessageDialog(getParentFrame(),"The compound could not be added, because it doesn't fullfil all criteria.");
+					JOptionPane.showMessageDialog(getParentFrame(),"The compound could not be added, because it doesn't "+reason+".");
 				}
 			}
 		else {
@@ -618,7 +636,7 @@ public class CompoundCollectionPane<T> extends JScrollPane
 				if (mCompoundFilter == null || mCompoundFilter.moleculeQualifies(mol))
 					mModel.setMolecule(mEditedIndex, mol);
 				else
-					JOptionPane.showMessageDialog(getParentFrame(),"The compound could not be changed, because the changed structure doesn't fullfil all criteria.");
+					JOptionPane.showMessageDialog(getParentFrame(),"The compound could not be changed, because the changed structure doesn't "+reason+".");
 				}
 			}
 		}
@@ -690,7 +708,8 @@ public class CompoundCollectionPane<T> extends JScrollPane
 							mModel.addMolecule(mDropIndex, mol);
 							}
 						else {
-							JOptionPane.showMessageDialog(getParentFrame(),"The compound could not be added, because it doesn't qualify.");
+							String reason = (mCompoundFilter instanceof SubstructureFilter) ? "match the substructure" : "qualify";
+							JOptionPane.showMessageDialog(getParentFrame(),"The compound could not be added, because it doesn't "+reason+".");
 							}
 						}
 					updateDropPosition(-1);
