@@ -43,7 +43,9 @@ import com.actelion.research.gui.dnd.MoleculeDropAdapter;
 import com.actelion.research.gui.dnd.MoleculeTransferable;
 import com.actelion.research.gui.dnd.ReactionDropAdapter;
 import com.actelion.research.gui.dnd.ReactionTransferable;
+import com.actelion.research.gui.generic.GenericDrawContext;
 import com.actelion.research.gui.hidpi.HiDPIHelper;
+import com.actelion.research.gui.swing.SwingDrawContext;
 import com.actelion.research.util.ColorHelper;
 import com.actelion.research.util.CursorHelper;
 
@@ -67,11 +69,13 @@ public class JChemistryView extends JComponent
 
 	private static final String ITEM_COPY_RXN = "Copy Reaction";
 	private static final String ITEM_PASTE_RXN = "Paste Reaction";
-	private static final String ITEM_OPEN_RXN = "Open Reaction File";
-	private static final String ITEM_SAVE_RXN = "Save Reaction File";
+	private static final String ITEM_OPEN_RXN = "Open Reaction File...";
+	private static final String ITEM_SAVE_RXN = "Save Reaction File...";
 	private static final String ITEM_COPY_MOLS = "Copy Molecules";
 	private static final String ITEM_PASTE_MOLS = "Paste Molecules";
 
+	private static final long WARNING_MILLIS = 1200;
+
 	private static final long serialVersionUID = 20150204L;
 	private static final int UPDATE_REDRAW_ONLY = 0;
 	private static final int UPDATE_CHECK_COORDS = 1;
@@ -82,6 +86,7 @@ public class JChemistryView extends JComponent
 	private static final int ALLOWED_DRAG_ACTIONS = DnDConstants.ACTION_COPY_OR_MOVE;
 	private static final int ALLOWED_DROP_ACTIONS = DnDConstants.ACTION_COPY_OR_MOVE;
 
+	private static final int DRAG_MARGIN = 12;
 	private static final int DRAG_TYPE_NONE = -1;
 	private static final int DRAG_TYPE_REACTION = -2;   // or molecule index >= 0
 
@@ -93,6 +98,7 @@ public class JChemistryView extends JComponent
 	private Color				mFragmentNoColor;
 	private MoleculeDropAdapter mMoleculeDropAdapter = null;
 	private ReactionDropAdapter mReactionDropAdapter = null;
+	private String              mWarningMessage;
 
 
 	/**
@@ -148,7 +154,7 @@ public class JChemistryView extends JComponent
 		}
 
 	public void setContent(StereoMolecule mol, DrawingObjectList drawingObjectList) {
-		mDepictor = new ExtendedDepictor(mol, drawingObjectList, true);
+		mDepictor = new ExtendedDepictor(mol, drawingObjectList);
 		mDepictor.setDisplayMode(mDisplayMode);
 		mDepictor.setFragmentNoColor(mFragmentNoColor);
 		mUpdateMode = UPDATE_SCALE_COORDS;
@@ -157,7 +163,7 @@ public class JChemistryView extends JComponent
 		}
 
 	public void setContent(StereoMolecule[] mol, DrawingObjectList drawingObjectList) {
-		mDepictor = new ExtendedDepictor(mol, drawingObjectList, true);
+		mDepictor = new ExtendedDepictor(mol, drawingObjectList);
 		mDepictor.setDisplayMode(mDisplayMode);
 		mDepictor.setFragmentNoColor(mFragmentNoColor);
 		mUpdateMode = UPDATE_SCALE_COORDS;
@@ -166,7 +172,7 @@ public class JChemistryView extends JComponent
 		}
 
 	public void setContent(Reaction rxn, DrawingObjectList drawingObjectList) {
-		mDepictor = new ExtendedDepictor(rxn, drawingObjectList, rxn == null ? false : rxn.isReactionLayoutRequired(), true);
+		mDepictor = new ExtendedDepictor(rxn, drawingObjectList, rxn == null ? false : rxn.isReactionLayoutRequired());
 		mDepictor.setDisplayMode(mDisplayMode);
 		mDepictor.setFragmentNoColor(mFragmentNoColor);
 		mUpdateMode = UPDATE_SCALE_COORDS;
@@ -245,15 +251,17 @@ public class JChemistryView extends JComponent
 		theSize.width -= insets.left + insets.right;
 		theSize.height -= insets.top + insets.bottom;
 
+		GenericDrawContext context = new SwingDrawContext((Graphics2D)g);
+
 		if (mSize == null
 		 || mSize.width != theSize.width
 		 || mSize.height != theSize.height
 		 || mUpdateMode == UPDATE_SCALE_COORDS) {
-			mDepictor.validateView(g, new Rectangle2D.Double(insets.left, insets.top, theSize.width, theSize.height),
+			mDepictor.validateView(context, new Rectangle2D.Double(insets.left, insets.top, theSize.width, theSize.height),
 									AbstractDepictor.cModeInflateToMaxAVBL);
 			}
 		else if (mUpdateMode == UPDATE_CHECK_COORDS) {
-			mDepictor.validateView(g, new Rectangle2D.Double(insets.left, insets.top, theSize.width, theSize.height), 0);
+			mDepictor.validateView(context, new Rectangle2D.Double(insets.left, insets.top, theSize.width, theSize.height), 0);
 			}
 
 		mSize = theSize;
@@ -264,34 +272,44 @@ public class JChemistryView extends JComponent
 		g2.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
 
 		Color fg = g2.getColor();
-		g2.setColor(UIManager.getColor(isEditable() && isEnabled() ? "TextField.background" : "TextField.inactiveBackground"));
+		Color bg = UIManager.getColor(isEditable() && isEnabled() ? "TextField.background" : "TextField.inactiveBackground");
+		g2.setColor(bg);
 		g2.fill(new Rectangle(insets.left, insets.top, theSize.width, theSize.height));
 		g2.setColor(fg);
 
 		mDepictor.setForegroundColor(getForeground(), getBackground());
 
 		if (mShowBorder && mDragType != DRAG_TYPE_NONE) {
-			Color bg = getBackground();
-			Color color = ColorHelper.perceivedBrightness(bg) < 0.5f ? bg.brighter() : bg.darker();
+			Color color = ColorHelper.perceivedBrightness(bg) < 0.5f ? ColorHelper.brighter(bg, 0.85f) : ColorHelper.darker(bg, 0.85f);
 			g.setColor(color);
 			Rectangle2D.Double rect = (mDragType == DRAG_TYPE_REACTION) ? getChemistryBounds() : getMoleculeBounds(mDragType);
-			Stroke oldStroke = ((Graphics2D)g).getStroke();
-			((Graphics2D)g).setStroke(new BasicStroke(HiDPIHelper.scale(2), BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
 			int arc = (int)Math.min(rect.height/4, Math.min(rect.width/4, HiDPIHelper.scale(10)));
-			g.drawRoundRect((int)rect.x, (int)rect.y, (int)rect.width, (int)rect.height, arc, arc);
-			((Graphics2D)g).setStroke(oldStroke);
+			g.fillRoundRect((int)rect.x, (int)rect.y, (int)rect.width, (int)rect.height, arc, arc);
 			if (mDragType == DRAG_TYPE_REACTION) {
 				final String msg = "<press 'ALT' to drag individual molecules>";
 				int fontSize = HiDPIHelper.scale(7);
+				g.setColor(fg);
 				g.setFont(getFont().deriveFont((float)fontSize));
 				int msgWidth = g.getFontMetrics().stringWidth(msg);
 				int x = (int)(rect.x+(rect.width-msgWidth)/2);
-				int y = (int)(rect.y+rect.height-fontSize/2);
+				int y = (int)Math.max(fontSize, rect.y - fontSize);
 				g.drawString(msg, x, y);
 				}
 			}
 
-		mDepictor.paint(g);
+		mDepictor.paint(context);
+
+		if (mWarningMessage != null) {
+			int fontSize = HiDPIHelper.scale(12);
+			g.setFont(getFont().deriveFont(Font.BOLD, (float)fontSize));
+			Color original = g.getColor();
+			g.setColor(Color.RED);
+			FontMetrics metrics = g.getFontMetrics();
+			Rectangle2D bounds = metrics.getStringBounds(mWarningMessage, g);
+			g.drawString(mWarningMessage, insets.left+(int)(theSize.width-bounds.getWidth())/2,
+					insets.top+metrics.getHeight());
+			g.setColor(original);
+			}
 
 		mUpdateMode = UPDATE_REDRAW_ONLY;
 		}
@@ -365,6 +383,8 @@ public class JChemistryView extends JComponent
 			StereoMolecule mol = ch.pasteMolecule();
 			if (mol != null)
 				pasteOrDropMolecule(mol);
+			else
+				showWarningMessage("No molecule on clipboard!");
 			}
 
 		if (e.getActionCommand().equals(ITEM_PASTE_RXN) && mIsEditable) {
@@ -372,6 +392,8 @@ public class JChemistryView extends JComponent
 			Reaction rxn = ch.pasteReaction();
 			if (rxn != null)
 				pasteOrDropReaction(rxn);
+			else
+				showWarningMessage("No reaction on clipboard!");
 			}
 
 		if (e.getActionCommand().equals(ITEM_OPEN_RXN) && mIsEditable) {
@@ -498,7 +520,7 @@ public class JChemistryView extends JComponent
 			boolean dragIndividualMolecule = mChemistryType != ExtendedDepictor.TYPE_REACTION || e.isAltDown();
 			if (dragIndividualMolecule) {
 				for (int i=0; i<mDepictor.getMoleculeCount(); i++) {
-					Rectangle2D.Double bounds = mDepictor.getMoleculeDepictor(i).getBoundingRect();
+					Rectangle bounds = shrink(mDepictor.getMoleculeDepictor(i).getBoundingRect());
 					if (bounds != null && bounds.contains(x, y)) {
 						dragType = i;
 						break;
@@ -506,7 +528,7 @@ public class JChemistryView extends JComponent
 					}
 				}
 			else {
-				Rectangle2D.Double bounds = getChemistryBounds();
+				Rectangle bounds = shrink(getChemistryBounds());
 				if (bounds != null && bounds.contains(x, y))
 					dragType = DRAG_TYPE_REACTION;
 				}
@@ -522,6 +544,16 @@ public class JChemistryView extends JComponent
 				CursorHelper.cPointerCursor : CursorHelper.cHandCursor));
 		}
 
+	private Rectangle shrink(Rectangle2D.Double rect) {
+		if (rect == null)
+			return null;
+
+		int margin = HiDPIHelper.scale(DRAG_MARGIN);
+		int marginX = Math.min(margin, (int)rect.width / 6);
+		int marginY = Math.min(margin, (int)rect.height / 6);
+		return new Rectangle((int)rect.x+marginX, (int)rect.y+marginY, (int)rect.width-2*marginX, (int)rect.height-2*marginY);
+		}
+
 	public void addStructureListener(StructureListener l) {
 		if(mListener == null)
 			mListener = new ArrayList<StructureListener>();
@@ -635,6 +667,16 @@ public class JChemistryView extends JComponent
 		informListeners();
 		}
 
+	protected void showWarningMessage(String msg) {
+		mWarningMessage = msg;
+		repaint();
+		new Thread(() -> {
+			try { Thread.sleep(WARNING_MILLIS); } catch (InterruptedException ie) {}
+			mWarningMessage = null;
+			repaint();
+			} ).start();
+		}
+
 	public Transferable getMoleculeTransferable() {
 		StereoMolecule mol = (mDragType < 0) ? null : mDepictor.getMolecule(mDragType).getCompactCopy();
 		return new MoleculeTransferable(mol);
