@@ -1,39 +1,55 @@
 /*
- * Copyright 2017 Idorsia Pharmaceuticals Ltd., Hegenheimermattweg 91, CH-4123 Allschwil, Switzerland
+ * Copyright (c) 1997 - 2016
+ * Actelion Pharmaceuticals Ltd.
+ * Gewerbestrasse 16
+ * CH-4123 Allschwil, Switzerland
  *
- * This file is part of DataWarrior.
- * 
- * DataWarrior is free software: you can redistribute it and/or modify it under the terms of the
- * GNU General Public License as published by the Free Software Foundation, either version 3 of
- * the License, or (at your option) any later version.
- * 
- * DataWarrior is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License along with DataWarrior.
- * If not, see http://www.gnu.org/licenses/.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright notice, this
+ *    list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
+ * 3. Neither the name of the the copyright holder nor the
+ *    names of its contributors may be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
+ * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+ * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  * @author Thomas Sander
  */
 
 package com.actelion.research.gui;
 
-import com.actelion.research.chem.AbstractDepictor;
-import com.actelion.research.chem.Depictor2D;
-import com.actelion.research.chem.IDCodeParser;
-import com.actelion.research.chem.StereoMolecule;
+import com.actelion.research.chem.*;
 import com.actelion.research.chem.name.StructureNameResolver;
 import com.actelion.research.gui.clipboard.IClipboardHandler;
 import com.actelion.research.gui.dnd.MoleculeDragAdapter;
 import com.actelion.research.gui.dnd.MoleculeDropAdapter;
 import com.actelion.research.gui.dnd.MoleculeTransferable;
+import com.actelion.research.gui.generic.GenericDepictor;
 import com.actelion.research.gui.hidpi.HiDPIHelper;
+import com.actelion.research.gui.swing.SwingDrawContext;
 import com.actelion.research.util.ColorHelper;
 import com.actelion.research.util.CursorHelper;
 
 import javax.swing.*;
 import java.awt.*;
-import java.awt.datatransfer.*;
+import java.awt.datatransfer.StringSelection;
+import java.awt.datatransfer.Transferable;
 import java.awt.dnd.*;
 import java.awt.event.*;
 import java.awt.geom.Rectangle2D;
@@ -43,22 +59,29 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
     static final long serialVersionUID = 0x20061113;
 
     private static final String ITEM_COPY = "Copy Structure";
+	private static final String ITEM_COPY_SMILES = "Copy Structure As SMILES-String";
 	private static final String ITEM_PASTE= "Paste Structure";
 	private static final String ITEM_PASTE_WITH_NAME = ITEM_PASTE+" or Name";
 	private static final String ITEM_CLEAR = "Clear Structure";
 
-    private ArrayList<StructureListener> mListener;
+	private static final long WARNING_MILLIS = 1200;
+
+	private static final int DRAG_MARGIN = 12;
+
+	private ArrayList<StructureListener> mListener;
 	private String mIDCode;
 	private StereoMolecule mMol,mDisplayMol;
-    private Depictor2D mDepictor;
-	private boolean mShowBorder,mAllowFragmentStatusChangeOnPasteOrDrop,mIsDraggingThis,mOpaqueBackground,
-					mIsEditable,mDisableBorder;
+    private GenericDepictor mDepictor;
+	private boolean mShowBorder,mAllowFragmentStatusChangeOnPasteOrDrop,mIsDraggingThis,mIsEditable,mDisableBorder;
 	private int mChiralTextPosition,mDisplayMode;
 	private String[] mAtomText;
 	private IClipboardHandler mClipboardHandler;
 	protected MoleculeDropAdapter mDropAdapter = null;
 	protected int mAllowedDragAction;
 	protected int mAllowedDropAction;
+	private String mWarningMessage;
+	private int[] mAtomHiliteColor;
+	private float[] mAtomHiliteRadius;
 
 	public JStructureView() {
         this(null);
@@ -129,11 +152,18 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 		mDisplayMol = (displayMol == null) ? mMol : displayMol;
 		mDisplayMode = AbstractDepictor.cDModeHiliteAllQueryFeatures;
 		mIsEditable = false;
+		updateBackground();
 		addMouseListener(this);
 		addMouseMotionListener(this);
 		initializeDragAndDrop(dragAction, dropAction);
 	    }
 
+	@Override
+	public void updateUI() {
+		super.updateUI();
+		updateBackground();
+		}
+
     /**
      * Call this in order to get clipboard support:
      * setClipboardHandler(new ClipboardHandler());
@@ -146,13 +176,20 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 		return mClipboardHandler;
 	    }
 
+	public int getDisplayMode() {
+		return mDisplayMode;
+		}
+
 	/**
 	 * Sets the display mode for the Depictor. The default is
 	 * AbstractDepictor.cDModeHiliteAllQueryFeatures.
 	 * @param mode
 	 */
 	public void setDisplayMode(int mode) {
-	    mDisplayMode = mode;
+		if (mDisplayMode != mode) {
+		    mDisplayMode = mode;
+		    repaint();
+			}
 	    }
 
 	public void setDisableBorder(boolean b) {
@@ -174,6 +211,7 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 
 	public void setEnabled(boolean enable) {
 		if (enable != isEnabled()) {
+			updateBackground();
 			repaint();
 			if (mDropAdapter != null)
 				mDropAdapter.setActive(enable);
@@ -201,6 +239,18 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 		mAllowFragmentStatusChangeOnPasteOrDrop = allow;
 		}
 
+	/**
+	 * If you want this structure view to also draw an atom background with specific colors for every atom,
+	 * then you need to call this method before or just after one of the structureChanged() calls.
+	 * @param argb if alpha < 1 then the background is mixed in accordingly
+	 * @param radius <= 1.0; if null, then a default of 0.5 of the average bond length is used
+	 */
+	public void setAtomHighlightColors(int[] argb, float[] radius) {
+		mAtomHiliteColor = argb;
+		mAtomHiliteRadius = radius;
+		repaint();
+		}
+
 	public boolean canDrop() {
 		return mIsEditable && isEnabled() && !mIsDraggingThis;
 	    }
@@ -223,38 +273,50 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
         g2.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
 
 		Color fg = g2.getColor();
-		g2.setColor(UIManager.getColor(isEditable() && isEnabled() ? "TextField.background" : "TextField.inactiveBackground"));
+		Color bg = getBackground();
+		g2.setColor(bg);
 		g2.fill(new Rectangle(insets.left, insets.top, theSize.width, theSize.height));
-		g2.setColor(fg);
-
-		if (mDisplayMol != null && mDisplayMol.getAllAtoms() != 0) {
-			mDepictor = new Depictor2D(mDisplayMol);
-            mDepictor.setDisplayMode(mDisplayMode);
-            mDepictor.setAtomText(mAtomText);
-
-			if (!isEnabled())
-                mDepictor.setOverruleColor(ColorHelper.getContrastColor(Color.GRAY, getBackground()), getBackground());
-			else
-				mDepictor.setForegroundColor(getForeground(), getBackground());
-
-			int avbl = HiDPIHelper.scale(AbstractDepictor.cOptAvBondLen);
-			mDepictor.validateView(g, new Rectangle2D.Double(insets.left, insets.top, theSize.width,theSize.height),
-								   AbstractDepictor.cModeInflateToMaxAVBL | mChiralTextPosition | avbl);
-            mDepictor.paint(g);
-			}
 
 		if (mShowBorder && !mDisableBorder) {
 			Rectangle2D.Double rect = mDepictor.getBoundingRect();
 			if (rect != null) {
-				Color bg = getBackground();
-				g.setColor(ColorHelper.perceivedBrightness(bg) < 0.5f ? bg.brighter() : bg.darker());
-				Stroke oldStroke = ((Graphics2D)g).getStroke();
-				((Graphics2D)g).setStroke(new BasicStroke(HiDPIHelper.scale(2), BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
+				g.setColor(ColorHelper.perceivedBrightness(bg) < 0.5f ? ColorHelper.brighter(bg, 0.85f) : ColorHelper.darker(bg, 0.85f));
 				int arc = (int)Math.min(rect.height/4, Math.min(rect.width/4, HiDPIHelper.scale(10)));
-				g.drawRoundRect((int)rect.x, (int)rect.y, (int)rect.width, (int)rect.height, arc, arc);
-				((Graphics2D)g).setStroke(oldStroke);
+				g.fillRoundRect((int)rect.x, (int)rect.y, (int)rect.width, (int)rect.height, arc, arc);
 				}
 			}
+
+		g2.setColor(fg);
+
+		if (mDisplayMol != null && mDisplayMol.getAllAtoms() != 0) {
+			mDepictor = new GenericDepictor(mDisplayMol);
+            mDepictor.setDisplayMode(mDisplayMode);
+            mDepictor.setAtomText(mAtomText);
+            mDepictor.setAtomHighlightColors(mAtomHiliteColor, mAtomHiliteRadius);
+
+			if (!isEnabled())
+                mDepictor.setOverruleColor(ColorHelper.getContrastColor(Color.GRAY, bg), bg);
+			else
+				mDepictor.setForegroundColor(getForeground(), bg);
+
+			int avbl = HiDPIHelper.scale(AbstractDepictor.cOptAvBondLen);
+			SwingDrawContext context = new SwingDrawContext((Graphics2D)g);
+			mDepictor.validateView(context, new Rectangle2D.Double(insets.left, insets.top, theSize.width,theSize.height),
+								   AbstractDepictor.cModeInflateToMaxAVBL | mChiralTextPosition | avbl);
+            mDepictor.paint(context);
+			}
+
+		if (mWarningMessage != null) {
+			int fontSize = HiDPIHelper.scale(12);
+			g.setFont(getFont().deriveFont(Font.BOLD, (float)fontSize));
+			Color original = g.getColor();
+			g.setColor(Color.RED);
+			FontMetrics metrics = g.getFontMetrics();
+			Rectangle2D bounds = metrics.getStringBounds(mWarningMessage, g);
+			g.drawString(mWarningMessage, insets.left+(int)(theSize.width-bounds.getWidth())/2,
+					insets.top+metrics.getHeight());
+			g.setColor(original);
+			}
 		}
 
 	public void setIDCode(String idcode) {
@@ -268,7 +330,7 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 		if (mIDCode == null && idcode == null)
 			return;
 
-		if (mIDCode != null && idcode != null && mIDCode.equals(idcode))
+		if (mIDCode != null && mIDCode.equals(idcode))
 			return;
 
 		new IDCodeParser(true).parse(mMol, idcode, coordinates);
@@ -374,7 +436,7 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 		int y = e.getY();
 		boolean isInRect = false;
 		if (mDepictor != null && (mAllowedDragAction & DnDConstants.ACTION_COPY) != 0) {
-			Rectangle2D.Double bounds = mDepictor.getBoundingRect();
+			Rectangle bounds = shrink(mDepictor.getBoundingRect());
 			if (bounds != null && bounds.contains(x, y))
 				isInRect = true;
 			}
@@ -383,14 +445,28 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 		setCursor(CursorHelper.getCursor(isInRect ? CursorHelper.cHandCursor : CursorHelper.cPointerCursor));
 		}
 
+	private Rectangle shrink(Rectangle2D.Double rect) {
+		int margin = HiDPIHelper.scale(DRAG_MARGIN);
+		int marginX = Math.min(margin, (int)rect.width / 6);
+		int marginY = Math.min(margin, (int)rect.height / 6);
+		return new Rectangle((int)rect.x+marginX, (int)rect.y+marginY, (int)rect.width-2*marginX, (int)rect.height-2*marginY);
+	}
+
 	@Override public void mouseDragged(MouseEvent e) {}
 
 	public void actionPerformed(ActionEvent e) {
 		if (e.getActionCommand().equals(ITEM_COPY)) {
 			mClipboardHandler.copyMolecule(mMol);
 			}
+		if (e.getActionCommand().equals(ITEM_COPY_SMILES)) {
+			final String smiles = new IsomericSmilesCreator(mMol).getSmiles();
+			final StringSelection data = new StringSelection(smiles);
+			Toolkit.getDefaultToolkit().getSystemClipboard().setContents(data, data);
+			}
 		if (e.getActionCommand().startsWith(ITEM_PASTE) && mIsEditable) {
-			StereoMolecule mol = mClipboardHandler.pasteMolecule();
+			int smartsMode = mAllowFragmentStatusChangeOnPasteOrDrop ? SmilesParser.SMARTS_MODE_GUESS
+						   : mMol.isFragment() ? SmilesParser.SMARTS_MODE_IS_SMARTS : SmilesParser.SMARTS_MODE_IS_SMILES;
+			StereoMolecule mol = mClipboardHandler.pasteMolecule(true, smartsMode);
 			if (mol != null) {
 				if (!mAllowFragmentStatusChangeOnPasteOrDrop)
 					mol.setFragment(mMol.isFragment());
@@ -398,14 +474,33 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 				mDisplayMol = mol;
 				structureChanged();
 				}
+			else {
+				showWarningMessage("No molecule on clipboard!");
+				}
 			}
 		if (e.getActionCommand().equals(ITEM_CLEAR) && mIsEditable) {
-			mMol.deleteMolecule();
+			mMol.clear();
 			mDisplayMol = mMol;
 			structureChanged();
 			}
 		}
 
+	protected void showWarningMessage(String msg) {
+		mWarningMessage = msg;
+		repaint();
+		new Thread(() -> {
+			try { Thread.sleep(WARNING_MILLIS); } catch (InterruptedException ie) {}
+			mWarningMessage = null;
+			repaint();
+			} ).start();
+		}
+
+	private void updateBackground() {
+		Color bg = UIManager.getColor(isEditable() && isEnabled() ? "TextField.background" : "TextField.inactiveBackground");
+		if (bg != null)
+			setBackground(bg);
+		}
+
 	private void handlePopupTrigger(MouseEvent e) {
 		if (mMol != null && e.isPopupTrigger() && mClipboardHandler != null) {
 			JPopupMenu popup = new JPopupMenu();
@@ -415,6 +510,11 @@ public class JStructureView extends JComponent implements ActionListener,MouseLi
 			item1.setEnabled(mMol.getAllAtoms() != 0);
 			popup.add(item1);
 
+			JMenuItem itemCopySmiles = new JMenuItem(ITEM_COPY_SMILES);
+			itemCopySmiles.addActionListener(this);
+			itemCopySmiles.setEnabled(mMol.getAllAtoms() != 0);
+			popup.add(itemCopySmiles);
+
 			if (mIsEditable) {
 				String itemText = StructureNameResolver.getInstance() == null ? ITEM_PASTE : ITEM_PASTE_WITH_NAME;
 				JMenuItem item2 = new JMenuItem(itemText);
