@@ -39,6 +39,8 @@ import com.actelion.research.chem.io.RXNFileCreator;
 import com.actelion.research.chem.io.RXNFileParser;
 import com.actelion.research.chem.name.StructureNameResolver;
 import com.actelion.research.chem.reaction.Reaction;
+import com.actelion.research.chem.reaction.ReactionEncoder;
+import com.actelion.research.gui.clipboard.external.ChemDrawCDX;
 import com.actelion.research.gui.dnd.MoleculeTransferable;
 import com.actelion.research.gui.dnd.ReactionTransferable;
 import com.actelion.research.gui.wmf.WMF;
@@ -78,25 +80,26 @@ public class ClipboardHandler implements IClipboardHandler
 	 */
 	@Override
 	public ArrayList<StereoMolecule> pasteMolecules() {
-		return pasteMolecules(true, true);
+		return pasteMolecules(true, true, SmilesParser.SMARTS_MODE_GUESS);
 	}
 
 	/**
 	 * Get one Molecule from the Clipboard. On all platforms the first choice is a serialized StereoMolecule.
 	 * Further supported formats on Windows are: MDLSK,MDLCT,MDL_MOL,CF_ENHMETAFILE with embedded sketch.
 	 * If no supported format is found and the clipboard contains text, which can be interpreted as molfile,
-	 * SMILES, IUPAC name or idcode, then the corresponding molecule is returned.
+	 * SMILES (with SMARTS features), IUPAC name or idcode, then the corresponding molecule is returned.
 	 * Otherwise, if a StructureNameResolver is present, then it tries to interpret the name.
 	 * If the clipboard molecule has 3D coordinates, then new 2D-coords are invented and used instead.
 	 * @return Molecule found or null if no molecule present on the clipboard
 	 */
 	@Override
 	public StereoMolecule pasteMolecule() {
-		return pasteMolecule(true);
+		return pasteMolecule(true, SmilesParser.SMARTS_MODE_GUESS);
 	}
 
-	public StereoMolecule pasteMolecule(boolean prefer2D) {
-		ArrayList<StereoMolecule> molList = pasteMolecules(prefer2D, false);
+	@Override
+	public StereoMolecule pasteMolecule(boolean prefer2D, int smartsMode) {
+		ArrayList<StereoMolecule> molList = pasteMolecules(prefer2D, false, smartsMode);
 		return molList.size() == 0 ? null : molList.get(0);
 	}
 
@@ -110,9 +113,10 @@ public class ClipboardHandler implements IClipboardHandler
 	 * corresponding molecules, which may be limited to a certain number.
 	 * @param prefer2D if true and if the clipboard molecule has 3D coordinates, then new 2D-coords are invented
 	 * @param allowMultiple whether multiple molecules may be generated from clipboard text, if no serialized mol or special molecule format present
+	 * @param smartsMode SmilesParser.SMARTS_MODE for parsing strings as SMILES, i.e. how SMARTS features are considered
 	 * @return list of molecules found or generated from SMILES, names, etc; empty list if no molecule present on the clipboard
 	 */
-	private ArrayList<StereoMolecule> pasteMolecules(boolean prefer2D, boolean allowMultiple) {
+	private ArrayList<StereoMolecule> pasteMolecules(boolean prefer2D, boolean allowMultiple, int smartsMode) {
 		ArrayList<StereoMolecule> molList = new ArrayList<>();
 
 		StereoMolecule mol = Platform.isWindows() ? pasteMoleculeWindowsNative(prefer2D) : pasteMoleculeLinux();
@@ -152,7 +156,7 @@ public class ClipboardHandler implements IClipboardHandler
 							if (mol == null || mol.getAllAtoms() == 0) {
 								mol = new StereoMolecule();
 								try {
-									new SmilesParser().parse(mol, line);
+									new SmilesParser(smartsMode, false).parse(mol, line);
 								}
 								catch (Exception e) {
 									mol = null;
@@ -292,9 +296,43 @@ public class ClipboardHandler implements IClipboardHandler
 	 * @return Reaction or null if no reaction present
 	 */
 	public Reaction pasteReaction() {
-		Reaction rxn;
+		Reaction rxn = Platform.isWindows() ? pasteReactionWindowsNative() : pasteReactionLinux();
 
-		return Platform.isWindows() ? pasteReactionWindowsNative() : pasteReactionLinux();
+		if (rxn == null) {
+			// get StringFlavor from clipboard and try parsing it as rxn-idcode, rxn-file, or reaction-smiles
+			Transferable t = Toolkit.getDefaultToolkit().getSystemClipboard().getContents(null);
+			String text = null;
+			try {
+				text = (String)t.getTransferData(DataFlavor.stringFlavor);
+			}
+			catch (Exception ioe) {}
+			if (text != null) {
+				try {
+					rxn = ReactionEncoder.decode(text, true);
+					if (rxn != null && rxn.isEmpty())
+						rxn = null;
+					}
+				catch (Exception e) {}
+				if (rxn == null) {
+					try {
+						rxn = new RXNFileParser().getReaction(text);
+						if (rxn != null && rxn.isEmpty())
+							rxn = null;
+					}
+					catch (Exception e) {}
+				}
+				if (rxn == null) {
+					try {
+						rxn = new SmilesParser().parseReaction(text);
+						if (rxn != null && rxn.isEmpty())
+							rxn = null;
+					}
+					catch (Exception e) {}
+				}
+			}
+		}
+
+		return rxn;
 	}
 
 	public Reaction pasteReactionWindowsNative() {
@@ -465,7 +503,8 @@ public class ClipboardHandler implements IClipboardHandler
 			ctab = mc.getRXNfile();
 		}
 
-		byte sketch[] = Sketch.createSketchFromReaction(makeRXNCopy(rxn));
+		ChemDrawCDX cdx = new com.actelion.research.gui.clipboard.external.ChemDrawCDX();
+		byte[] cdxBuffer = cdx.getChemDrawBuffer(rxn);
 
 		// Serialize to a byte array
 		System.out.println("copyReactionToClipboard");
@@ -476,7 +515,7 @@ public class ClipboardHandler implements IClipboardHandler
 		out.close();
 		bos.close();
 
-		return NativeClipboardAccessor.copyReactionToClipboard(ctab.getBytes(), sketch, bos.toByteArray());
+		return NativeClipboardAccessor.copyReactionToClipboard(ctab.getBytes(), cdxBuffer, bos.toByteArray());
 	}
 
 	private boolean writeMol2Metafile(File temp, StereoMolecule m, byte[] sketch) {
